// @HEADER
// ***********************************************************************
//
//           Panzer: A partial differential equation assembly
//       engine for strongly coupled complex multiphysics systems
//                 Copyright (2011) Sandia Corporation
//
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Roger P. Pawlowski (rppawlo@sandia.gov) and
// Eric C. Cyr (eccyr@sandia.gov)
// ***********************************************************************
// @HEADER


#ifndef PANZER_DIMENSION_HPP
#define PANZER_DIMENSION_HPP

#include "Phalanx_ExtentTraits.hpp"

namespace panzer {
  /*
  //! Spatial Dimension Tag
  SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(Dim)
  //! Integration Point Tag
  SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(IP)
  //! Basis Point Tag (generalization of the NODE)
  SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(BASIS)

  //! Node Point Tag
  // SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(NODE)
  using NODE = BASIS;

  //! Generic Point Tag
  SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(Point)
  //! Cell Tag
  SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(Cell)
  //! Face Tag
  SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(Face)
  //! Dummy Tag
  SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(Dummy)
  //! Edge Tag
  SHARDS_ARRAY_DIM_TAG_SIMPLE_DECLARATION(Edge)  // Suzey: 06/11/201
  */

  struct Dim{};
  struct IP{};
  struct BASIS{};
  using NODE = BASIS;
  struct Point{};
  struct Cell{};
  struct Face{};
  struct Edge{};
  struct Dummy{};
}

namespace PHX {
  template<> std::string print<panzer::Dim>();
  template<> std::string print<panzer::IP>();
  template<> std::string print<panzer::BASIS>();
  template<> std::string print<panzer::Point>();
  template<> std::string print<panzer::Cell>();
  template<> std::string print<panzer::Face>();
  template<> std::string print<panzer::Edge>();
  template<> std::string print<panzer::Dummy>();
}

PHX_IS_EXTENT(panzer::Dim)
PHX_IS_EXTENT(panzer::IP)
PHX_IS_EXTENT(panzer::BASIS)
PHX_IS_EXTENT(panzer::Point)
PHX_IS_EXTENT(panzer::Cell)
PHX_IS_EXTENT(panzer::Face)
PHX_IS_EXTENT(panzer::Edge)
PHX_IS_EXTENT(panzer::Dummy)

#endif
