# @HEADER
# ************************************************************************
#
#            TriBITS: Tribal Build, Integrate, and Test System
#                    Copyright 2013 Sandia Corporation
#
# Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
# the U.S. Government retains certain rights in this software.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the Corporation nor the names of the
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/STKBalance clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "STKBalance requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/STKBalance build
## ---------------------------------------------------------------------------

set(STKBalance_CXX_COMPILER "/opt/aarch64-linux-gnu/aarch64-linux-gnu/sys-root/usr/local/bin/mpicxx")

set(STKBalance_C_COMPILER "/opt/aarch64-linux-gnu/aarch64-linux-gnu/sys-root/usr/local/bin/mpicc")

set(STKBalance_Fortran_COMPILER "/opt/bin/aarch64-linux-gnu-libgfortran5-cxx03-mpi+mpich/aarch64-linux-gnu-gfortran")
# Deprecated!
set(STKBalance_FORTRAN_COMPILER "/opt/bin/aarch64-linux-gnu-libgfortran5-cxx03-mpi+mpich/aarch64-linux-gnu-gfortran") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/STKBalance build
## ---------------------------------------------------------------------------

## Give the build type
set(STKBalance_CMAKE_BUILD_TYPE "Release")

## Set compiler flags, including those determined by build type
set(STKBalance_CXX_FLAGS [[ ]])

set(STKBalance_C_FLAGS [[  -O3 -fPIC -fopenmp ]])

set(STKBalance_Fortran_FLAGS [[ ]])
# Deprecated
set(STKBalance_FORTRAN_FLAGS [[ ]])

## Extra link flags (e.g., specification of fortran libraries)
set(STKBalance_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(STKBalance_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/workspace/destdir/lib")
set(STKBalance_BUILD_SHARED_LIBS "ON")

set(STKBalance_LINKER /opt/bin/aarch64-linux-gnu-libgfortran5-cxx03-mpi+mpich/aarch64-linux-gnu-ld)
set(STKBalance_AR /opt/bin/aarch64-linux-gnu-libgfortran5-cxx03-mpi+mpich/aarch64-linux-gnu-ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(STKBalance_INSTALL_DIR "/workspace/destdir")

## List of package libraries
set(STKBalance_LIBRARIES STKBalance::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(STKBalance_MPI_LIBRARIES "")
set(STKBalance_MPI_LIBRARY_DIRS "")
set(STKBalance_MPI_INCLUDE_DIRS "")
set(STKBalance_MPI_EXEC "/opt/aarch64-linux-gnu/aarch64-linux-gnu/sys-root/usr/local/bin/mpiexec")
set(STKBalance_MPI_EXEC_MAX_NUMPROCS "4")
set(STKBalance_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(STKBalance_ENABLE_TeuchosCore ON)
set(STKBalance_ENABLE_TeuchosParameterList ON)
set(STKBalance_ENABLE_SEACASNemesis ON)
set(STKBalance_ENABLE_SEACASExodus ON)
set(STKBalance_ENABLE_SEACASIoss ON)
set(STKBalance_ENABLE_Zoltan2Core ON)
set(STKBalance_ENABLE_STKUtil ON)
set(STKBalance_ENABLE_STKSearch ON)
set(STKBalance_ENABLE_STKMesh ON)
set(STKBalance_ENABLE_STKIO ON)
set(STKBalance_ENABLE_STKTools ON)
set(STKBalance_ENABLE_STKTopology ON)
set(STKBalance_ENABLE_BLAS ON)
set(STKBalance_ENABLE_LAPACK ON)

# Exported cache variables
set(STK_ENABLE_DEBUG "OFF")
set(HAVE_STK_DEBUG "OFF")
set(STK_Trilinos "ON")
set(HAVE_STK_Trilinos "ON")

# Include configuration of dependent packages
if (NOT TARGET TeuchosCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosCore/TeuchosCoreConfig.cmake")
endif()
if (NOT TARGET TeuchosParameterList::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosParameterList/TeuchosParameterListConfig.cmake")
endif()
if (NOT TARGET SEACASNemesis::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASNemesis/SEACASNemesisConfig.cmake")
endif()
if (NOT TARGET SEACASExodus::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASExodus/SEACASExodusConfig.cmake")
endif()
if (NOT TARGET SEACASIoss::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASIoss/SEACASIossConfig.cmake")
endif()
if (NOT TARGET Zoltan2Core::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Zoltan2Core/Zoltan2CoreConfig.cmake")
endif()
if (NOT TARGET STKUtil::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKUtil/STKUtilConfig.cmake")
endif()
if (NOT TARGET STKSearch::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKSearch/STKSearchConfig.cmake")
endif()
if (NOT TARGET STKMesh::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKMesh/STKMeshConfig.cmake")
endif()
if (NOT TARGET STKIO::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKIO/STKIOConfig.cmake")
endif()
if (NOT TARGET STKTools::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTools/STKToolsConfig.cmake")
endif()
if (NOT TARGET STKTopology::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTopology/STKTopologyConfig.cmake")
endif()
if (NOT TARGET BLAS::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/BLAS/BLASConfig.cmake")
endif()
if (NOT TARGET LAPACK::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/LAPACK/LAPACKConfig.cmake")
endif()

# Import STKBalance targets
include("${CMAKE_CURRENT_LIST_DIR}/STKBalanceTargets.cmake")

# Standard TriBITS-compliant external package variables
set(STKBalance_IS_TRIBITS_COMPLIANT TRUE)
set(STKBalance_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(STKBalance_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(STKBalance_EXPORTED_PACKAGE_LIBS_NAMES "stk_balance_lib")

foreach(libname IN LISTS STKBalance_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE STKBalance::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'STKBalance::${libname}', or better yet,"
      " 'STKBalance::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'STKBalance'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'STKBalance_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
