# @HEADER
# ************************************************************************
#
#            TriBITS: Tribal Build, Integrate, and Test System
#                    Copyright 2013 Sandia Corporation
#
# Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
# the U.S. Government retains certain rights in this software.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the Corporation nor the names of the
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/STKMesh clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "STKMesh requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/STKMesh build
## ---------------------------------------------------------------------------

set(STKMesh_CXX_COMPILER "/opt/aarch64-linux-gnu/aarch64-linux-gnu/sys-root/usr/local/bin/mpicxx")

set(STKMesh_C_COMPILER "/opt/aarch64-linux-gnu/aarch64-linux-gnu/sys-root/usr/local/bin/mpicc")

set(STKMesh_Fortran_COMPILER "/opt/bin/aarch64-linux-gnu-libgfortran5-cxx03-mpi+mpich/aarch64-linux-gnu-gfortran")
# Deprecated!
set(STKMesh_FORTRAN_COMPILER "/opt/bin/aarch64-linux-gnu-libgfortran5-cxx03-mpi+mpich/aarch64-linux-gnu-gfortran") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/STKMesh build
## ---------------------------------------------------------------------------

## Give the build type
set(STKMesh_CMAKE_BUILD_TYPE "Release")

## Set compiler flags, including those determined by build type
set(STKMesh_CXX_FLAGS [[ ]])

set(STKMesh_C_FLAGS [[  -O3 -fPIC -fopenmp ]])

set(STKMesh_Fortran_FLAGS [[ ]])
# Deprecated
set(STKMesh_FORTRAN_FLAGS [[ ]])

## Extra link flags (e.g., specification of fortran libraries)
set(STKMesh_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(STKMesh_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/workspace/destdir/lib")
set(STKMesh_BUILD_SHARED_LIBS "ON")

set(STKMesh_LINKER /opt/bin/aarch64-linux-gnu-libgfortran5-cxx03-mpi+mpich/aarch64-linux-gnu-ld)
set(STKMesh_AR /opt/bin/aarch64-linux-gnu-libgfortran5-cxx03-mpi+mpich/aarch64-linux-gnu-ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(STKMesh_INSTALL_DIR "/workspace/destdir")

## List of package libraries
set(STKMesh_LIBRARIES STKMesh::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(STKMesh_MPI_LIBRARIES "")
set(STKMesh_MPI_LIBRARY_DIRS "")
set(STKMesh_MPI_INCLUDE_DIRS "")
set(STKMesh_MPI_EXEC "/opt/aarch64-linux-gnu/aarch64-linux-gnu/sys-root/usr/local/bin/mpiexec")
set(STKMesh_MPI_EXEC_MAX_NUMPROCS "4")
set(STKMesh_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(STKMesh_ENABLE_Shards ON)
set(STKMesh_ENABLE_STKTopology ON)
set(STKMesh_ENABLE_STKUtil ON)
set(STKMesh_ENABLE_Kokkos ON)
set(STKMesh_ENABLE_BLAS ON)
set(STKMesh_ENABLE_MPI ON)

# Exported cache variables
set(STK_ENABLE_DEBUG "OFF")
set(HAVE_STK_DEBUG "OFF")
set(STK_Trilinos "ON")
set(HAVE_STK_Trilinos "ON")

# Include configuration of dependent packages
if (NOT TARGET Shards::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Shards/ShardsConfig.cmake")
endif()
if (NOT TARGET STKTopology::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTopology/STKTopologyConfig.cmake")
endif()
if (NOT TARGET STKUtil::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKUtil/STKUtilConfig.cmake")
endif()
if (NOT TARGET Kokkos::all_libs)
  include("/opt/aarch64-linux-gnu/aarch64-linux-gnu/sys-root/usr/local/lib/cmake/Kokkos/KokkosConfig.cmake")
  set(Kokkos_CONFIG "/opt/aarch64-linux-gnu/aarch64-linux-gnu/sys-root/usr/local/lib/cmake/Kokkos/KokkosConfig.cmake")
endif()
if (NOT TARGET BLAS::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/BLAS/BLASConfig.cmake")
endif()
if (NOT TARGET MPI::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/MPI/MPIConfig.cmake")
endif()

# Import STKMesh targets
include("${CMAKE_CURRENT_LIST_DIR}/STKMeshTargets.cmake")

# Standard TriBITS-compliant external package variables
set(STKMesh_IS_TRIBITS_COMPLIANT TRUE)
set(STKMesh_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(STKMesh_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(STKMesh_EXPORTED_PACKAGE_LIBS_NAMES "stk_mesh_base")

foreach(libname IN LISTS STKMesh_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE STKMesh::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'STKMesh::${libname}', or better yet,"
      " 'STKMesh::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'STKMesh'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'STKMesh_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
